@@ -10,10 +10,11 @@ use crate::color::{FromColor, Luma, LumaA, Rgb, Rgba};
 use crate::dynimage::{save_buffer, save_buffer_with_format, write_buffer_with_format};
 use crate::error::ImageResult;
 use crate::flat::{FlatSamples, SampleLayout};
-use crate::image::{GenericImage, GenericImageView, ImageEncoder, ImageFormat, ImageOutputFormat};
+use crate::image::{GenericImage, GenericImageView, ImageEncoder, ImageFormat};
 use crate::math::Rect;
 use crate::traits::{EncodableLayout, Pixel, PixelWithColorType};
 use crate::utils::expand_packed;
+use crate::DynamicImage;
 
 /// Iterate over pixel refs.
 pub struct Pixels<'a, P: Pixel + 'a>
@@ -643,7 +644,7 @@ where
 /// image::imageops::overlay(&mut img, &on_top, 128, 128);
 /// ```
 ///
-/// Convert an RgbaImage to a GrayImage.
+/// Convert an `RgbaImage` to a `GrayImage`.
 ///
 /// ```no_run
 /// use image::{open, DynamicImage};
@@ -789,7 +790,7 @@ where
             .saturating_mul(num_channels);
 
         self.data
-            .get(i..i + num_channels)
+            .get(i..i.checked_add(num_channels)?)
             .map(|pixel_indices| <P as Pixel>::from_slice(pixel_indices))
     }
 
@@ -800,7 +801,7 @@ where
     /// the bounds will not overflow.
     fn check_image_fits(width: u32, height: u32, len: usize) -> bool {
         let checked_len = Self::image_buffer_len(width, height);
-        checked_len.map(|min_len| min_len <= len).unwrap_or(false)
+        checked_len.is_some_and(|min_len| min_len <= len)
     }
 
     fn image_buffer_len(width: u32, height: u32) -> Option<usize> {
@@ -888,7 +889,7 @@ where
     Container: Deref<Target = [P::Subpixel]> + DerefMut,
 {
     // TODO: choose name under which to expose.
-    fn inner_pixels_mut(&mut self) -> &mut [P::Subpixel] {
+    pub(crate) fn inner_pixels_mut(&mut self) -> &mut [P::Subpixel] {
         let len = Self::image_buffer_len(self.width, self.height).unwrap();
         &mut self.data[..len]
     }
@@ -967,7 +968,7 @@ where
             .saturating_mul(num_channels);
 
         self.data
-            .get_mut(i..i + num_channels)
+            .get_mut(i..i.checked_add(num_channels)?)
             .map(|pixel_indices| <P as Pixel>::from_slice_mut(pixel_indices))
     }
 
@@ -979,7 +980,7 @@ where
     #[inline]
     #[track_caller]
     pub fn put_pixel(&mut self, x: u32, y: u32, pixel: P) {
-        *self.get_pixel_mut(x, y) = pixel
+        *self.get_pixel_mut(x, y) = pixel;
     }
 }
 
@@ -1043,15 +1044,11 @@ where
 {
     /// Writes the buffer to a writer in the specified format.
     ///
-    /// Assumes the writer is buffered. In most cases,
-    /// you should wrap your writer in a `BufWriter` for best performance.
-    ///
-    /// See [`ImageOutputFormat`](enum.ImageOutputFormat.html) for
-    /// supported types.
-    pub fn write_to<W, F>(&self, writer: &mut W, format: F) -> ImageResult<()>
+    /// Assumes the writer is buffered. In most cases, you should wrap your writer in a `BufWriter`
+    /// for best performance.
+    pub fn write_to<W>(&self, writer: &mut W, format: ImageFormat) -> ImageResult<()>
     where
         W: std::io::Write + std::io::Seek,
-        F: Into<ImageOutputFormat>,
         P: PixelWithColorType,
     {
         // This is valid as the subpixel is u8.
@@ -1160,6 +1157,12 @@ where
             _phantom: PhantomData,
         }
     }
+
+    fn clone_from(&mut self, source: &Self) {
+        self.data.clone_from(&source.data);
+        self.width = source.width;
+        self.height = source.height;
+    }
 }
 
 impl<P, Container> GenericImageView for ImageBuffer<P, Container>
@@ -1173,10 +1176,6 @@ where
         self.dimensions()
     }
 
-    fn bounds(&self) -> (u32, u32, u32, u32) {
-        (0, 0, self.width, self.height)
-    }
-
     fn get_pixel(&self, x: u32, y: u32) -> P {
         *self.get_pixel(x, y)
     }
@@ -1199,7 +1198,7 @@ where
     }
 
     fn put_pixel(&mut self, x: u32, y: u32, pixel: P) {
-        *self.get_pixel_mut(x, y) = pixel
+        *self.get_pixel_mut(x, y) = pixel;
     }
 
     /// Puts a pixel at location (x, y), ignoring bounds checking.
@@ -1207,14 +1206,14 @@ where
     unsafe fn unsafe_put_pixel(&mut self, x: u32, y: u32, pixel: P) {
         let indices = self.pixel_indices_unchecked(x, y);
         let p = <P as Pixel>::from_slice_mut(self.data.get_unchecked_mut(indices));
-        *p = pixel
+        *p = pixel;
     }
 
     /// Put a pixel at location (x, y), taking into account alpha channels
     ///
     /// DEPRECATED: This method will be removed. Blend the pixel directly instead.
     fn blend_pixel(&mut self, x: u32, y: u32, p: P) {
-        self.get_pixel_mut(x, y).blend(&p)
+        self.get_pixel_mut(x, y).blend(&p);
     }
 
     fn copy_within(&mut self, source: Rect, x: u32, y: u32) -> bool {
@@ -1264,9 +1263,12 @@ where
 impl<P: Pixel> ImageBuffer<P, Vec<P::Subpixel>> {
     /// Creates a new image buffer based on a `Vec<P::Subpixel>`.
     ///
+    /// all the pixels of this image have a value of zero, regardless of the data type or number of channels.
+    ///
     /// # Panics
     ///
-    /// Panics when the resulting image is larger the the maximum size of a vector.
+    /// Panics when the resulting image is larger than the maximum size of a vector.
+    #[must_use]
     pub fn new(width: u32, height: u32) -> ImageBuffer<P, Vec<P::Subpixel>> {
         let size = Self::image_buffer_len(width, height)
             .expect("Buffer length in `ImageBuffer::new` overflows usize");
@@ -1278,39 +1280,40 @@ impl<P: Pixel> ImageBuffer<P, Vec<P::Subpixel>> {
         }
     }
 
-    /// Constructs a new ImageBuffer by copying a pixel
+    /// Constructs a new `ImageBuffer` by copying a pixel
     ///
     /// # Panics
     ///
-    /// Panics when the resulting image is larger the the maximum size of a vector.
+    /// Panics when the resulting image is larger than the maximum size of a vector.
     pub fn from_pixel(width: u32, height: u32, pixel: P) -> ImageBuffer<P, Vec<P::Subpixel>> {
         let mut buf = ImageBuffer::new(width, height);
         for p in buf.pixels_mut() {
-            *p = pixel
+            *p = pixel;
         }
         buf
     }
 
-    /// Constructs a new ImageBuffer by repeated application of the supplied function.
+    /// Constructs a new `ImageBuffer` by repeated application of the supplied function.
     ///
     /// The arguments to the function are the pixel's x and y coordinates.
     ///
     /// # Panics
     ///
-    /// Panics when the resulting image is larger the the maximum size of a vector.
+    /// Panics when the resulting image is larger than the maximum size of a vector.
     pub fn from_fn<F>(width: u32, height: u32, mut f: F) -> ImageBuffer<P, Vec<P::Subpixel>>
     where
         F: FnMut(u32, u32) -> P,
     {
         let mut buf = ImageBuffer::new(width, height);
         for (x, y, p) in buf.enumerate_pixels_mut() {
-            *p = f(x, y)
+            *p = f(x, y);
         }
         buf
     }
 
     /// Creates an image buffer out of an existing buffer.
     /// Returns None if the buffer is not big enough.
+    #[must_use]
     pub fn from_vec(
         width: u32,
         height: u32,
@@ -1321,6 +1324,7 @@ impl<P: Pixel> ImageBuffer<P, Vec<P::Subpixel>> {
 
     /// Consumes the image buffer and returns the underlying data
     /// as an owned buffer
+    #[must_use]
     pub fn into_vec(self) -> Vec<P::Subpixel> {
         self.into_raw()
     }
@@ -1340,6 +1344,7 @@ impl GrayImage {
     /// Expands a color palette by re-using the existing buffer.
     /// Assumes 8 bit per pixel. Uses an optionally transparent index to
     /// adjust it's alpha value accordingly.
+    #[must_use]
     pub fn expand_palette(
         self,
         palette: &[(u8, u8, u8)],
@@ -1396,7 +1401,7 @@ where
         let mut buffer: ImageBuffer<ToType, Vec<ToType::Subpixel>> =
             ImageBuffer::new(self.width, self.height);
         for (to, from) in buffer.pixels_mut().zip(self.pixels()) {
-            to.from_color(from)
+            to.from_color(from);
         }
         buffer
     }
@@ -1427,21 +1432,102 @@ pub type Rgb32FImage = ImageBuffer<Rgb<f32>, Vec<f32>>;
 /// where the backing container is a flattened vector of floats.
 pub type Rgba32FImage = ImageBuffer<Rgba<f32>, Vec<f32>>;
 
+impl From<DynamicImage> for RgbImage {
+    fn from(value: DynamicImage) -> Self {
+        value.into_rgb8()
+    }
+}
+
+impl From<DynamicImage> for RgbaImage {
+    fn from(value: DynamicImage) -> Self {
+        value.into_rgba8()
+    }
+}
+
+impl From<DynamicImage> for GrayImage {
+    fn from(value: DynamicImage) -> Self {
+        value.into_luma8()
+    }
+}
+
+impl From<DynamicImage> for GrayAlphaImage {
+    fn from(value: DynamicImage) -> Self {
+        value.into_luma_alpha8()
+    }
+}
+
+impl From<DynamicImage> for Rgb16Image {
+    fn from(value: DynamicImage) -> Self {
+        value.into_rgb16()
+    }
+}
+
+impl From<DynamicImage> for Rgba16Image {
+    fn from(value: DynamicImage) -> Self {
+        value.into_rgba16()
+    }
+}
+
+impl From<DynamicImage> for Gray16Image {
+    fn from(value: DynamicImage) -> Self {
+        value.into_luma16()
+    }
+}
+
+impl From<DynamicImage> for GrayAlpha16Image {
+    fn from(value: DynamicImage) -> Self {
+        value.into_luma_alpha16()
+    }
+}
+
+impl From<DynamicImage> for Rgba32FImage {
+    fn from(value: DynamicImage) -> Self {
+        value.into_rgba32f()
+    }
+}
+
 #[cfg(test)]
 mod test {
-    use super::{GrayImage, ImageBuffer, ImageOutputFormat, RgbImage};
+    use super::{GrayImage, ImageBuffer, RgbImage};
     use crate::math::Rect;
     use crate::GenericImage as _;
-    use crate::{color, Rgb};
+    use crate::ImageFormat;
+    use crate::{Luma, LumaA, Pixel, Rgb, Rgba};
+    use num_traits::Zero;
 
     #[test]
     /// Tests if image buffers from slices work
     fn slice_buffer() {
         let data = [0; 9];
-        let buf: ImageBuffer<color::Luma<u8>, _> = ImageBuffer::from_raw(3, 3, &data[..]).unwrap();
-        assert_eq!(&*buf, &data[..])
+        let buf: ImageBuffer<Luma<u8>, _> = ImageBuffer::from_raw(3, 3, &data[..]).unwrap();
+        assert_eq!(&*buf, &data[..]);
     }
 
+    macro_rules! new_buffer_zero_test {
+        ($test_name:ident, $pxt:ty) => {
+            #[test]
+            fn $test_name() {
+                let buffer = ImageBuffer::<$pxt, Vec<<$pxt as Pixel>::Subpixel>>::new(2, 2);
+                assert!(buffer
+                    .iter()
+                    .all(|p| *p == <$pxt as Pixel>::Subpixel::zero()));
+            }
+        };
+    }
+
+    new_buffer_zero_test!(luma_u8_zero_test, Luma<u8>);
+    new_buffer_zero_test!(luma_u16_zero_test, Luma<u16>);
+    new_buffer_zero_test!(luma_f32_zero_test, Luma<f32>);
+    new_buffer_zero_test!(luma_a_u8_zero_test, LumaA<u8>);
+    new_buffer_zero_test!(luma_a_u16_zero_test, LumaA<u16>);
+    new_buffer_zero_test!(luma_a_f32_zero_test, LumaA<f32>);
+    new_buffer_zero_test!(rgb_u8_zero_test, Rgb<u8>);
+    new_buffer_zero_test!(rgb_u16_zero_test, Rgb<u16>);
+    new_buffer_zero_test!(rgb_f32_zero_test, Rgb<f32>);
+    new_buffer_zero_test!(rgb_a_u8_zero_test, Rgba<u8>);
+    new_buffer_zero_test!(rgb_a_u16_zero_test, Rgba<u16>);
+    new_buffer_zero_test!(rgb_a_f32_zero_test, Rgba<f32>);
+
     #[test]
     fn get_pixel() {
         let mut a: RgbImage = ImageBuffer::new(10, 10);
@@ -1449,13 +1535,13 @@ mod test {
             let b = a.get_mut(3 * 10).unwrap();
             *b = 255;
         }
-        assert_eq!(a.get_pixel(0, 1)[0], 255)
+        assert_eq!(a.get_pixel(0, 1)[0], 255);
     }
 
     #[test]
     fn get_pixel_checked() {
         let mut a: RgbImage = ImageBuffer::new(10, 10);
-        a.get_pixel_mut_checked(0, 1).map(|b| b[0] = 255);
+        a.get_pixel_mut_checked(0, 1).unwrap()[0] = 255;
 
         assert_eq!(a.get_pixel_checked(0, 1), Some(&Rgb([255, 0, 0])));
         assert_eq!(a.get_pixel_checked(0, 1).unwrap(), a.get_pixel(0, 1));
@@ -1480,7 +1566,7 @@ mod test {
             let val = a.pixels_mut().next().unwrap();
             *val = Rgb([42, 0, 0]);
         }
-        assert_eq!(a.data[0], 42)
+        assert_eq!(a.data[0], 42);
     }
 
     #[test]
@@ -1547,12 +1633,8 @@ mod test {
 
     #[test]
     fn test_image_buffer_copy_within_tl() {
-        let data = &[
-            00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15,
-        ];
-        let expected = [
-            00, 01, 02, 03, 04, 00, 01, 02, 08, 04, 05, 06, 12, 08, 09, 10,
-        ];
+        let data = &[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
+        let expected = [0, 1, 2, 3, 4, 0, 1, 2, 8, 4, 5, 6, 12, 8, 9, 10];
         let mut image: GrayImage = ImageBuffer::from_raw(4, 4, Vec::from(&data[..])).unwrap();
         assert!(image.copy_within(
             Rect {
@@ -1569,12 +1651,8 @@ mod test {
 
     #[test]
     fn test_image_buffer_copy_within_tr() {
-        let data = &[
-            00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15,
-        ];
-        let expected = [
-            00, 01, 02, 03, 01, 02, 03, 07, 05, 06, 07, 11, 09, 10, 11, 15,
-        ];
+        let data = &[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
+        let expected = [0, 1, 2, 3, 1, 2, 3, 7, 5, 6, 7, 11, 9, 10, 11, 15];
         let mut image: GrayImage = ImageBuffer::from_raw(4, 4, Vec::from(&data[..])).unwrap();
         assert!(image.copy_within(
             Rect {
@@ -1591,12 +1669,8 @@ mod test {
 
     #[test]
     fn test_image_buffer_copy_within_bl() {
-        let data = &[
-            00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15,
-        ];
-        let expected = [
-            00, 04, 05, 06, 04, 08, 09, 10, 08, 12, 13, 14, 12, 13, 14, 15,
-        ];
+        let data = &[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
+        let expected = [0, 4, 5, 6, 4, 8, 9, 10, 8, 12, 13, 14, 12, 13, 14, 15];
         let mut image: GrayImage = ImageBuffer::from_raw(4, 4, Vec::from(&data[..])).unwrap();
         assert!(image.copy_within(
             Rect {
@@ -1613,12 +1687,8 @@ mod test {
 
     #[test]
     fn test_image_buffer_copy_within_br() {
-        let data = &[
-            00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15,
-        ];
-        let expected = [
-            05, 06, 07, 03, 09, 10, 11, 07, 13, 14, 15, 11, 12, 13, 14, 15,
-        ];
+        let data = &[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
+        let expected = [5, 6, 7, 3, 9, 10, 11, 7, 13, 14, 15, 11, 12, 13, 14, 15];
         let mut image: GrayImage = ImageBuffer::from_raw(4, 4, Vec::from(&data[..])).unwrap();
         assert!(image.copy_within(
             Rect {
@@ -1637,9 +1707,10 @@ mod test {
     #[cfg(feature = "png")]
     fn write_to_with_large_buffer() {
         // A buffer of 1 pixel, padded to 4 bytes as would be common in, e.g. BMP.
+
         let img: GrayImage = ImageBuffer::from_raw(1, 1, vec![0u8; 4]).unwrap();
         let mut buffer = std::io::Cursor::new(vec![]);
-        assert!(img.write_to(&mut buffer, ImageOutputFormat::Png).is_ok());
+        assert!(img.write_to(&mut buffer, ImageFormat::Png).is_ok());
     }
 
     #[test]
@@ -1690,8 +1761,6 @@ mod test {
 #[cfg(feature = "benchmarks")]
 mod benchmarks {
     use super::{ConvertBuffer, GrayImage, ImageBuffer, Pixel, RgbImage};
-    use crate::GenericImage;
-    use test;
 
     #[bench]
     fn conversion(b: &mut test::Bencher) {
